var env = require('./env');

module.exports = {	
	eqp: eqp,
	neqp: neqp,
	eq: eq,
	neq: neq,
	gt: gt,
	gte: gte,
	lt: lt,
	lte: lte,
	equalp: equalp,
	arrayeq: arrayeq,
	hasheq: hasheq,
	numberp: numberp,
	stringp: stringp,
	undefinedp: undefinedp,
	nullp: undefinedp,
	arrayp: arrayp,
	functionp: functionp,
	hashp: hashp,	
	slice: slice,
	plus: plus,
	minus: minus,
	mult: mult,
	div: div,
	def: def,
	get: get,
	set: set,
	len: len,
	cons: cons,
	car: car,
	cdr: cdr,
	first: first,
	last: last,
	pop: pop,
	map: map,
	reduce: reduce,
	assign: assign
};

function numberp(num){
	return typeof num == "number";
}

function stringp(str){
	return typeof str == "string";
}

function undefinedp(und){
	return typeof und == "undefined";
}

function arrayp(arr){
	return arr instanceof Array;
}

function functionp(fun){
	return fun instanceof Function;
}

function hashp(has){
	return has instanceof Object &&
		!(arrayp(has)) &&
			!(functionp(has));
}

function slice(arr, index){
	return Array.prototype.slice.call(arr, index);
}

function plus(){
	return slice(arguments).reduce(function(a,b){
		return a + b;
	});
}

function minus(){
	var args = slice(arguments);
	if (args.length === 1) args.unshift(0);
	return args.reduce(function(a,b){
		return a - b;
	});
}

function mult(){
	return slice(arguments).reduce(function(a,b){
		return a * b;
	});
}

function div(){
	return slice(arguments).reduce(function(a,b){
		return a / b;
	});
}

function def(env, key, value){
	return (env.values[key] = value);
}

function get(hash, key, deflt){
	if (hashp(hash))
		if (hash.hasOwnProperty(key))
			return hash[key];
		else return deflt;
}

function set(hash, key, value){
	if (hashp(hash))
		if (hash.hasOwnProperty(key))
			return hash[key] = value;
}

function eqp(a, b){
	return a === b;
}

function neqp(a, b){
	return a !== b;
}

function equalp(a, b){
	if (typeof(a) != typeof(b))
		return false;
	if (arrayp(a))
		return arrayp(b) && 
			arrayeq(a, b);
	if (hashp(a))
		return hashp(b) &&
			hasheq(a, b);
	return a == b;
}

function arrayeq(a, b){
	if (a.length != b.length)
		return false;
	for (var i=0,l=a.length;i<l;i++)
		if (a[i] != b[i])
			return false;
	return true;
}

function hasheq(a, b){
	return arrayeq(env.keys(a),env.keys(b)) &&
		arrayeq(values(a),values(b));
}

function eq(a, b){
	return a == b;
}

function neq(a, b){
	return a != b;
}

function gt(a, b){
	return a > b;
}

function gte(a, b){
	return a >= b;
}

function lt(a, b){
	return a < b;
}

function lte(a, b){
	return a <= b;
}

function nth(arr, n){
	if (arrayp(arr))
		return arr[n];
}

function len(a){
	if (hashp(a))
		return len(env.keys(a));
	return a.length;
}

function cons(a, as){
	return [a].concat(as);
}

function car(x){
	if (x.length) return x[0];
}

function cdr(x){
	if (x.length > 1) return x.slice(1);
}

function first(a){
	return a[0];
}

function last(a){
	return a[a.length-1];
}

function pop(a){
	return a.pop();
}

function map(arr, fn){
	return arr.map(fn);
}

function reduce(arr, fn, init){
	return arr.reduce(function(a, b){
		return fn(a, b);			
	},init);
}

function assign(keys, args){
	var _keys = [], _args = [];
    for (var k=0,a=0,l=args.length;a<l;a++,k++){
        if (keys[k] && keys[k].match(/^&/)){
            var nextk = keys.length - k - 1;
            _keys.push(keys[k].replace(/^&/,''));
            if (nextk == 0){ 
                _args.push(args.slice(a)); 
                break; 
            } else { 
                var slot = args.slice(a,args.length - nextk);
                _args.push(slot); a += slot.length - 1;
            }
        }                                                                                                                                                             
        _keys.push(keys[k]);
        _args.push(args[a]);
    }
	return [_keys, _args];
}
